// ---------------------------------------------------------------------------
// file: Deconvolute.java
// Expression Deconvolution, for
// Reconstructing cell populations from DNA microarray data
//
// Edward Marcotte
// Institute for Cellular and Molecular Biology,
// Department of Chemistry and Biochemistry, &
// Center for Computational Biology and Bioinformatics
// University of Texas at Austin
// marcotte@icmb.utexas.edu
//
// last updated: Tue Jan 14 08:11:00 CST 2003
// ---------------------------------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import java.text.*;

public class Deconvolute extends JTextArea
{

    double [][] basisvector, expvector;
     Hashtable expnamehash;
     String [] basislabel, explabel, basisname, expname;
     int linecnt=0, wordcnt=0, linenum = 0, size[], loopmax, trial;
     int basiscnt, numbasis, expcnt, len = 0;
     double [] testvector, targetvector, raw, x0raw, x0perc, perc, xminperc, xminraw, x0, xmin;
     double sum, score;
     boolean basisFlag, expFlag;
     File basisFile, expFile;

   public final AppendingTextArea outputArea;
   public final JMenu DeconMenu;

   public static void main(String[] args) {

        Deconvolute mw = new Deconvolute();
   }

   public Deconvolute() {
     JFrame frame = new JFrame( "Expression Deconvolution");
     frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE);
     //setBounds( 200, 200, 950, 750);
     frame.setSize( 900, 700);
     frame.getContentPane().setBackground(Color.white);
     frame.setVisible(true);

     JMenu FileMenu = new JMenu("File");
	 DeconMenu = new JMenu ("Deconvolute");
	 JMenu HelpMenu = new JMenu("Help");
     JMenu QuitMenu = new JMenu("Quit");

	 JMenuItem openBasisItem = new JMenuItem("Open Basis Sets");
	 	 openBasisItem.setMnemonic(KeyEvent.VK_B);
	 	 openBasisItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.CTRL_MASK));
	 	 openBasisItem.addActionListener(new ActionListener() {
	 	    public void actionPerformed(ActionEvent e) {
				outputArea.append("\n\n*** Opening DNA microarray data for use as basis sets***\n");
				openBasisOuter();
				}
	 	 });
	 FileMenu.add(openBasisItem);

	 JMenuItem openExpItem = new JMenuItem("Open Expression Sets");
	 	 openExpItem.setMnemonic(KeyEvent.VK_E);
	 	 openExpItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, Event.CTRL_MASK));
	 	 openExpItem.addActionListener(new ActionListener() {
	 	    public void actionPerformed(ActionEvent e) {
	     		outputArea.append("\n\n*** Opening DNA microarray data to be fit as a linear combination of the basis sets ***\n");
				openExpOuter();
				}
	 	 });
	 FileMenu.add(openExpItem);

	 JMenuItem openDeconItem = new JMenuItem("Deconvolute");
	 	 openDeconItem.setMnemonic(KeyEvent.VK_D);
	 	 openDeconItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, Event.CTRL_MASK));
	 	 openDeconItem.addActionListener(new ActionListener() {
	 	    public void actionPerformed(ActionEvent e) {
				outputArea.append("\n\n\n*** Deconvoluting the expression data ***\n");
				deconvolute();
				}
	 	 });
	 DeconMenu.add(openDeconItem);
     if (basisFlag && expFlag) {DeconMenu.setEnabled(true); repaint();}
         else {DeconMenu.setEnabled(false);}

	 JMenuItem openHelpItem = new JMenuItem("Help");
	 	 openHelpItem.setMnemonic(KeyEvent.VK_H);
	 	 openHelpItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, Event.CTRL_MASK));
	 	 openHelpItem.addActionListener(new ActionListener() {
	 	    public void actionPerformed(ActionEvent e) { help();}
	 	 });
	 HelpMenu.add(openHelpItem);

	 HelpMenu.addSeparator();

	 JMenuItem openAboutItem = new JMenuItem("About Deconvolute...");
	 	 openAboutItem.setMnemonic(KeyEvent.VK_A);
	 	 openAboutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, Event.CTRL_MASK));
	 	 openAboutItem.addActionListener(new ActionListener() {
	 	    public void actionPerformed(ActionEvent e) { showAboutBox();}
	 	 });
	 HelpMenu.add(openAboutItem);

	 JMenuItem quitItem = new JMenuItem("Quit");
	 	  quitItem.setMnemonic(KeyEvent.VK_Q);
	      quitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK));
	      quitItem.addActionListener(new ActionListener() {
	 		 public void actionPerformed(ActionEvent e) { System.exit(0);}
	     });
	 QuitMenu.add(quitItem);

	 JMenuBar menuBar = new JMenuBar ();
	 menuBar.add(FileMenu);
	 menuBar.add(DeconMenu);
	 menuBar.add(HelpMenu);
	 menuBar.add(QuitMenu);
     frame.setJMenuBar(menuBar);

	 outputArea = new AppendingTextArea();
	 outputArea.setColumns(80);
	 outputArea.setRows(40);
	 outputArea.setEditable(false);

	 final JScrollPane pane = new JScrollPane(outputArea);
	 pane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	 pane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	 frame.getContentPane().add(pane);
	 frame.pack();

	 outputArea.appendText("\n\n            ** Welcome to Expression Deconvolution! **\n\n");
	 outputArea.appendText("Load basis and expression data, then select \"Deconvolute\" to begin.\n");
	 outputArea.appendText("Information about file formats, calculations, etc. are available under the \"Help\" menu.\n\n");

}


     public void openBasisOuter() {
	     int linenum;
	     openBasis();
	     //outputArea.appendText("Read in " + basiscnt + " rows, " + numbasis + " columns from file " + basisFile + ".\n");
         outputArea.appendText("----------------------------------------------------------------------------------------------------------\n");
         if (basisFlag && expFlag) {DeconMenu.setEnabled(true); repaint();}
         else {DeconMenu.setEnabled(false);}
	     repaint();
     }


     public void openExpOuter() {
   	     int linenum;
   	     openExp();
   	     //outputArea.appendText("Read in " + expcnt + " rows, " + len + "columns from file " + expFile + ".\n");
   	     outputArea.appendText("----------------------------------------------------------------------------------------------------------\n");
   	     if (basisFlag && expFlag) {DeconMenu.setEnabled(true); repaint();}
         else {DeconMenu.setEnabled(false);}
   	     repaint();
     }



     public int[] getFileSize(File filename) {
	   int wordcnt =0, colcnt = 0, rowcnt = 0, size[];
	   String line, word;
	   size = new int [2];
	   try
	   {
	     BufferedReader in = new BufferedReader(new FileReader(filename));
	     while((line = in.readLine()) != null)
	       {
	          StringTokenizer st = new StringTokenizer(line, "\t", true);   // previously space & tab
	   	      wordcnt=0;
	   	      while ( st.hasMoreTokens() )
	   		    {
	   		       word = null;
	   		       //while ( (word == null) || ( word.equals(" ")) || (word.equals("\t") ) )
	   		       while ( (word == null) || (word.equals("\t") ) )
	   		   	     {
	   		 		    word = st.nextToken();
	   		 	     }
	   		       wordcnt++;
	   		    }
	   	      if (wordcnt > colcnt) colcnt = wordcnt;
	   	      if (wordcnt > 0)
	   	        rowcnt++;
	       }
	   }
	   catch (Exception e)
	   {
	     outputArea.appendText( "\nCannot open file: " + e+"\n");
	     repaint();
	   }
	   size[0] = colcnt;
	   size[1] = rowcnt;
	   return size;
     }



   synchronized private void openBasis() {
     // Open the basis expression data sets
     //outputArea.appendText("*** Opening DNA microarray data for use as basis sets***\n");

     String line, word;
     int tempnum, tempnum2;
	 double [] tempbvector, tempbvector2;

     JFileChooser chooser = new JFileChooser();
     int result = chooser.showOpenDialog(this);
     if (result == JFileChooser.CANCEL_OPTION) return;
     try
     {
		 basisFile = chooser.getSelectedFile();
		 size = getFileSize(basisFile);
		 numbasis = size[0] - 1;            // # of basis vectors (e.g. cell populations)
		 basiscnt = size[1] - 1;            // # genes in each basis vector
		 basisvector = new double [basiscnt][numbasis];
		 basisname = new String [basiscnt];    // gene names
		 basislabel = new String [numbasis];   // basis vector experiment names
		 tempbvector = new double [basiscnt];
		 tempbvector2 = new double [basiscnt];
		 linecnt = 0;
		 tempnum=0;
		 tempnum2=0;

		 //now read in the file
		 BufferedReader in = new BufferedReader(new FileReader(basisFile));
		 while((line = in.readLine()) != null)
		   {
		     StringTokenizer st = new StringTokenizer(line, "\t", true);  // previously space & tab
		     wordcnt=0;
		     while ( st.hasMoreTokens() )
		       {
		 		 word = null;
		 		 //while ( (word == null) || ( word.equals(" ")) || (word.equals("\t") ) )
		 		 while ( (word == null) || (word.equals("\t") ) )
		 		   {
		 		      word = st.nextToken();
		 		   }
		         if (linecnt==0)
		           {
					 if (wordcnt==0)
					  	{
						   // skip UNIQID

				 		}
				 	 else
				 	 	{
						  // read in basisvector names, arrayed in basislabel[]
						  tempnum = wordcnt-1;
		       		  	  basislabel[tempnum] = word;
		       			  //didFirst = true;
						}
			       }
		         else
		           {
					   if (wordcnt==0)
					   	 {
					   	   // read in gene name, arrayed in basisname[]
					       tempnum = linecnt - 1;
					       basisname[tempnum] = word;
					   	 }
					   else
					   	 {
					   	   // read in basis data point
					   	   tempnum = linecnt - 1;
					   	   tempnum2 = wordcnt - 1;
					   	   basisvector[tempnum][tempnum2] = Double.parseDouble(word);
						}
				   }
		         wordcnt++;
		       }
             linecnt++;
	 	   }

	    outputArea.appendText("\n----------------------------------------------------------------------------------------------------------\n");
	    outputArea.appendText("Read " + numbasis + " basis vectors ("+basiscnt+"x"+numbasis+" data matrix from file "+basisFile+"\n");
		outputArea.appendText("Basis vectors: ");
		for (int i=0; i < numbasis; i++)
		   {
			 outputArea.appendText( basislabel[i] + " ");
	   	   }
	   	outputArea.appendText("\n");
		outputArea.appendText("For example, here are 5 randomly chosen data points:\n");
		Random rnums = new Random();
	 	for (int i=0; i < 5; i++)
		   {
			 tempnum = rnums.nextInt(basiscnt);
		     tempnum2 = rnums.nextInt(numbasis);
		     outputArea.appendText( "  Gene " + tempnum+" ( "+basisname[tempnum]+" ) in "+ basislabel[tempnum2]+" data is " + basisvector[tempnum][tempnum2]+"\n");
		   }

		//normalize each basis vector
		for (int i = 0; i< numbasis; i++)
		 {
   	       //double [] tempbvector = new double [];
     	   for (int j = 0; j<basiscnt; j++)
       	   	  {
             	 tempbvector[j] = basisvector[j][i];
       		  }
     	   tempbvector2 =  normalizeVector(tempbvector);
     	   for (int j = 0; j<basiscnt; j++)
       		  {
          		 basisvector[j][i] = tempbvector2[j];
       		  }
 		 }
 	    outputArea.appendText("Basis vectors have been normalized\n");
		basisFlag = true;
     }
	 catch (Exception e)
	 {
		 outputArea.appendText("\nCannot open file: " +e+"\n");
		 repaint();
	 }
   }



   synchronized private void openExp() {
     // Open the expression data to be analyzed
     //outputArea.appendText("*** Opening DNA microarray data to be fit as a linear combination of the basis sets ***\n");

     String line, word;
     int tempnum, tempnum2;

     JFileChooser chooser = new JFileChooser();
     int result = chooser.showOpenDialog(this);
     if (result == JFileChooser.CANCEL_OPTION) return;
     try
     {
		 expFile = chooser.getSelectedFile();
		 size = getFileSize(expFile);
		 len = size[0] - 1;
		 expcnt = size[1] - 1;
		 expvector = new double [expcnt][len];
		 expname = new String [expcnt];        // gene names
		 expnamehash = new Hashtable();        // gene name hash
		 explabel = new String [len];
		 linecnt = 0;
		 tempnum = 0;
		 tempnum2 = 0;

		 //now read in the file
		 BufferedReader in = new BufferedReader(new FileReader(expFile));
		 while((line = in.readLine()) != null)
		   {
		     StringTokenizer st = new StringTokenizer(line, "\t", true);  // previously space & tab
		     wordcnt=0;
		     while ( st.hasMoreTokens() )
		       {
		 		 word = null;
		 		 //while ( (word == null) || ( word.equals(" ")) || (word.equals("\t") ) )
		 		 while ( (word == null) || (word.equals("\t") ) )
		 		   {
		 		      word = st.nextToken();
		 		   }
		 		 //if (word.equals("String"))
		         //System.out.println( linecnt + "-" + wordcnt + ":" + line);
		         if (linecnt==0)
				   {
					 if (wordcnt==0)
				   	   {
				 	     // skip UNIQID
				       }
				 	 else
				 	   {
				 	     // read in expvector names, arrayed in explabel[]
				 		 tempnum = wordcnt-1;
				 		 explabel[tempnum] = word;
				 	   }
				   }
				 else
				   {
				 	 if (wordcnt==0)
				 	   {
				 		 // read in gene name, arrayed in expname[]
				 		 tempnum = linecnt - 1;
				 		 expname[tempnum] = word;
				 		 expnamehash.put((Object) word, new Integer(tempnum));
				 		 //System.out.println("Key:"+word+" Val:"+expnamehash.get((Object) word));
				 	   }
				 	 else
				 	   {
				 		 // read in exp data point
				 		 tempnum = linecnt - 1;
				 		 tempnum2 = wordcnt - 1;
				 		 expvector[tempnum][tempnum2] = Double.parseDouble(word);
				 	   }
				 	}
		         wordcnt++;
		       }
		     linecnt++;
	 	   }

   	     outputArea.appendText("\n----------------------------------------------------------------------------------------------------------\n");
	     outputArea.appendText("Read " + len + " expression data sets ("+expcnt+"x"+len+" data matrix from file "+expFile+"\n");
		 outputArea.appendText("For example, here are 5 randomly chosen data points:\n");
	     Random rnums = new Random();
	     for (int i=0; i < 5; i++)
	       {
		     tempnum = rnums.nextInt(expcnt);
		     tempnum2 = rnums.nextInt(len);
		     outputArea.appendText( "  Gene " + tempnum+" ( "+expname[tempnum]+" ) in "+ explabel[tempnum2]+" data is " + expvector[tempnum][tempnum2]+"\n");
		   }
     expFlag = true;
     }
	 catch (Exception e)
	 {
		 outputArea.appendText("\nCannot open file: "+e+"\n");
		 repaint();
	 }
   }


public void deconvolute() {
	 // Deconvolute all loaded expression data
	 int loopmax = (int) 10;   // num divisions of each basis probability during optimization
     raw = new double [numbasis];
     targetvector = new double [basiscnt];
     testvector = new double [basiscnt];

	 //outputArea.appendText("\n\n\n*** Deconvoluting the expression data ***\n");

     repaint(  ); // Paint again so we can see the change.

	 OUTER:
     for (int expcolumn = 0; expcolumn < len; expcolumn++)
       {
		   outputArea.appendText("\n\n----------------------------------------------------------------------------------------------------------\n");
		   outputArea.appendText("Fitting " + explabel[expcolumn] + " data (column " + expcolumn + ")\n");

		   // First, extract subset of genes corresponding to the basis vectors into a separate vector
		   int numna = 0;
		   double zerosum = 0;
		   EXPCOL:
		   for (int i = 0; i < basiscnt; i++)   // EXPCOL
		     {

				Object o = expnamehash.get(basisname[i]);
				if (o == null) {targetvector[i] = (double) -999; numna++; continue EXPCOL;}
				Integer counts = (Integer) o;
                targetvector[i] = expvector[counts.intValue()][expcolumn];
                if (targetvector[i] == (double) -999) {numna++;}
                   else { zerosum += targetvector[i];}
			 }
		   if (numna == basiscnt)
		      {
				  outputArea.appendText("Skipping this one.  Too many data missing.\n----------------------------------------------------------------------------------------------------------\n");
				  continue OUTER;
		      }
      	   if (zerosum == 0)
      	      {
				  outputArea.appendText("Skipping this one.  zerosum==0.\n----------------------------------------------------------------------------------------------------------\n");
				  continue OUTER;
			  }
		   targetvector = normalizeVector(targetvector);


		   // Generate random initial starting vector for optimization
		   Random rnums = new Random();
		   for (int i=0; i< numbasis; i++) {raw[i]=rnums.nextInt(loopmax);}
           sum = (double) 0;
           score = (double) 0;
           for (int i=0; i<numbasis; i++) {sum+=raw[i];}
           if (sum == (double) 0) sum = (double) 1;
           perc = new double [numbasis];
           for (int i=0; i<numbasis; i++)
             {
                perc[i]=raw[i]/sum;
             }
           testvector = new double [basiscnt];
           CHK1:
           for (int i=0; i<basiscnt; i++)
             {
                testvector[i] = (double) 0;
                for (int j=0; j<numbasis; j++)
                  {
                     if (basisvector[i][j] == (double) -999) {testvector[i]= (double) -999;  continue CHK1;}
                     testvector[i] += perc[j] * basisvector[i][j];
                  }
             }
           testvector = normalizeVector(testvector);
           score = -1 * corrcoeff();

 		   // Solve for optimal linear combination of basis vectors
           simulatedAnnealing();

	   }
   }


   synchronized private void help() {
     // Help
     //setForeground(Color.red);
     //outputArea.appendText("Help with file formats, interpretation, etc...");
     //repaint(  ); // Paint again so we can see the change.

     JFrame helpme = new ExpDeconHelp();
     helpme.setVisible(true);
   }



   public void quit() {
         System.exit(0);
   }


   public double corrcoeff() {
   // Calculate correlation coefficient (Pearson's) between two vectors
   // corrcoeff = corrcoeff();   Note: uses global variables @testvector @targetvector $basiscnt
     double sim = 0, sumx = 0, sumy = 0, sumxsq = 0, sumysq = 0, sumxy = 0;
     int n = 0;
	 //outputArea.appendText("Called corrcoeff routine\n");

     SKIP:
     for (int k = 0; k < basiscnt; k++)
             {
                if ((testvector[k] == (double) -999) || (targetvector[k] == (double) -999))
                  continue SKIP;
                    sumx += testvector[k];
                    sumy += targetvector[k];
                    sumxy += (testvector[k] * targetvector[k]);
                    sumxsq += (testvector[k] * testvector[k]);
                    sumysq += (targetvector[k] * targetvector[k]);
                    n++;
             }
     sim = (sumxy - ((sumx*sumy)/ (double) n)) /
          Math.sqrt( (sumxsq - ((sumx*sumx) / (double) n)) * (sumysq - ((sumy*sumy)/ (double) n)) );
     return sim;
   }

   public double [] normalizeVector(double [] tempvector) {
     // subroutine to normalize vector to length 1
	 //double [] tempvector;
	 double sum = 0;
	 //outputArea.appendText("Called normalization routine\n");
	 NORM:
	 for (int i=0; i<basiscnt; i++)
	    {
	       if (tempvector[i] == (double) -999) continue NORM;
	       sum+= (tempvector[i]*tempvector[i]);
	    }
	 if (sum==0)
	   {
		   outputArea.appendText("***ERROR*** zero length vector sent to normalization routine\n\n");
		   //die;
	   }
	 sum = Math.sqrt(sum);
	 NORM2:
	 for (int i = 0; i < basiscnt; i++)
	   {
	     if (tempvector[i] == (double) -999) continue NORM2;
	     tempvector[i] = tempvector[i] / sum;
	   }
	 return tempvector;
   }


public void simulatedAnnealing () {
     // simulated annealing optimization of linear combination of vectors
 	 // adapted from computational science education project code
	 // uses global variables & prints output directly
	 outputArea.appendText("Performing simulated annealing fit...\n");
	 int nt = (int) 15000;    // Max num trials   1500
	 int ltran = (int) 1000;  // Max num trials at a given temp  500
	 int maccs = (int) 250;  // Max num acceptances at a given temp   250
	 double t1 = (double) 0.01, t, dfave; //  0.01       // initial temp.  If 0, optimize
	 double alfa = (double) 0.95;    // Cooling rate control parameter


	 // Total number of objective function increases generated
	 int inx = (int) 0;
	 //int inxs = (int) 0;

	 // Number of objective function increases accepted

	 int inc = (int) 0;
	 int incs = (int) 0;

	 // Number of objective function reductions generated

	 int red = (int) 0;
	 int reds = (int) 0;

	 // Number of absolute objective function reductions found
	 int ard = (int) 0;
	 int ards = (int) 0;

	 // Number of trials since successful one -
	 // increase or decrease of objective function
	 // Number of consecutive 'complete failures'
	 int rep = (int) 0;

	 int its = (int) 0;  // Number of trials at current temperature
	 double rat = (double) 0;
	 double ratmin = 0.01;   // minimum acceptance ratio  0.1

	 double of = (double) -1 * corrcoeff();      // find initial value of objective function
	 //outputArea.appendText( "Initial score (-1 * correlation coefficient) = " + of+"\n");
	 MessageFormat mfis = new MessageFormat(
	   "Initial score (Pearson correlation coefficient) = {0,number, #.#####}");
	 Object[] argumentsis = {new Double(-1 * of)};
	 outputArea.appendText(mfis.format(argumentsis)+"\n");



	 // Store starting point as best solution so far
	 x0 = new double [basiscnt];
	 xmin = new double [basiscnt];

	 for (int i=0; i < basiscnt; i++)
	  {
	        x0[i] = testvector[i];
	        xmin[i] = testvector[i];
	  }
	 x0perc = new double [numbasis];
	 x0raw = new double [numbasis];
	 xminperc = new double [numbasis];
	 xminraw = new double [numbasis];
	 for (int i=0; i < numbasis; i++)
	  {
	        x0perc[i] = perc[i];
	        x0raw[i] = raw[i];
	        xminperc[i] = perc[i];
	        xminraw[i] = raw[i];
	  }
	 double of0 = of;
	 double ofmin = of;
	 double sdof, dof;

	 // If initial temperature estimation is required, conduct a 100 trial survey

	 if (t1 == 0)
	  {
	    //outputArea.appendText( "Choosing initial temperature...\n");
	    sdof=0;
	    for (trial=0; trial<100; trial++)
	      {

	           // reset to previous condition
	           for (int i=0; i<basiscnt; i++)
	            {
	                testvector[i] = x0[i];}
	           for (int i=0; i < numbasis; i++)
	            {
	                perc[i] = x0perc[i];
	                raw[i] = x0raw[i];
	            }

	           // random change in proportions
	           Random rnums = new Random();
	           sum = (double) 0;
	           while (sum == (double) 0)
	             {
	               int randcol1 = rnums.nextInt(numbasis);
	               double randinc = 1;
	           	   if (rnums.nextDouble() < 0) randinc = (double) -1;
	               if ((raw[randcol1] + randinc) > (double) loopmax) randinc = (double) -1;
	               if ((raw[randcol1] + randinc) < 0) randinc = 1 ;
	               raw[randcol1] = raw[randcol1] + randinc;
	               sum = 0;
	               for (int i=0; i<numbasis; i++) {sum+=raw[i];}
			     }
	           for (int i=0; i<numbasis; i++)
	             {
	                perc[i]=raw[i]/sum;
	             }
	           CHK2:
	           for (int i=0; i<basiscnt; i++)
	             {
	                testvector[i] = 0;
	                for (int j=0; j<numbasis; j++)
	                  {
	                     if (basisvector[i][j] == (double) -999) {testvector[i] = (double) -999;  continue CHK2;}
	                     testvector[i] += perc[j] * basisvector[i][j];
	                  }
	             }
	           testvector = normalizeVector(testvector);


	           // Evaluate objective function after swaps
	           of = (double) -1 * corrcoeff();

	           // If the objective function is the smallest so far
	           if (of < ofmin)
	             {
	                // Then update stores
	                ofmin = of;

	                for (int i=0; i<basiscnt; i++)
	                  {
	                         xmin[i] = testvector[i];
	                  }
	                for (int i=0; i < numbasis; i++)
	                  {
	                      xminperc[i] = perc[i];
	                      xminraw[i] = raw[i];
	                  }
	                ard++;
	             }


	           // If the objective is increased, update statistics
	           if (of > of0)
	             {
	                inx++;
	                dof = of - of0;
	                sdof = sdof + dof;
	             }

	           // In this initial survey, accept all changes
	           for (int i=0; i< basiscnt; i++)
	             {
	                    x0[i] = testvector[i];
	              }
	           for (int i=0; i < numbasis; i++)
	              {
	                    x0perc[i] = perc[i];
	                    x0raw[i] = raw[i];
	              }
	           of0 = of;
	     }

	   // At end of survey calculate appropriate initial temperature
	   // (To give initial acceptance ratio of 0.8)
	   // (ln(1.25) = 0.22314)

	   dfave = sdof / (double) inx;
	   t1 = (double) 0.22314 * dfave;
	   //t1 = (double) 10 * t1;          // init estimate seems far too conservative
	   inx = 0;
	   //outputArea.appendText("Initial temp = " + t1+"\n");
	   //MessageFormat mft = new MessageFormat(
	   // "Initial temp = {0,number, ####.####}");
	   //Object[] arguments = {new Double(t1)};
	   //outputArea.appendText(mft.format(arguments)+"\n");


	 }


	 // Start of algorithm itself
	 // Set initial temperature

    outputArea.appendText("Optimizing...\nN\tOFO\tOFMAX\tINC\tRED\tARD\tT\tRAT\n");


	 t = t1;

     OUT:
	 for ( trial=1; trial < nt; trial++)
	   {
	     its++;

	     // reset to previous condition
	     for (int i=0; i<basiscnt; i++)
	      {
	          testvector[i] = x0[i];}
	     for (int i=0; i < numbasis; i++)
	       {
	          perc[i] = x0perc[i];
	          raw[i] = x0raw[i];
	       }

	     // random change in proportions
	     Random rnums = new Random();

		 sum = (double) 0;
	     while (sum == (double) 0)
	       {
			   int randcol1 = rnums.nextInt(numbasis);
	           double randinc = 1;

			   if (rnums.nextDouble() < 0) randinc = (double) -1;
			   if ((raw[randcol1] + randinc) > (double) loopmax) randinc = (double) -1;
			   if ((raw[randcol1] + randinc) < 0) randinc = 1 ;
			   raw[randcol1] = raw[randcol1] + randinc;
			   sum = 0;
	           for (int i=0; i<numbasis; i++) {sum+=raw[i];}
		   }
	     for (int i=0; i<numbasis; i++)
	       {
	           perc[i]=raw[i]/sum;
	       }
	           //@testvector=();
	     CHK3:
	     for (int i=0; i<basiscnt; i++)
	       {
	           testvector[i] = 0;
	           for (int j=0; j<numbasis; j++)
	             {
	                if (basisvector[i][j] == (double) -999) {testvector[i]= (double)-999;  continue CHK3;}
	                testvector[i] += perc[j] * basisvector[i][j];
	             }
	       }
	     testvector = normalizeVector(testvector);

	     // Evaluate objective function after swaps
	     of = (double) -1 * corrcoeff();

	     // If the objective function is the smallest so far
	     if (of < ofmin)
	       {
	           // Then update stores
	           ofmin = of;
 			   for (int i=0; i<basiscnt; i++)
                  {
                     xmin[i] = testvector[i];
                  }
                for (int i=0; i < numbasis; i++)
                   {
                     xminperc[i] = perc[i];
                     xminraw[i] = raw[i];
                   }
                ard++;
           }

	     // If objective function is reduced from current value, accept new position
	     if (of < of0)
	             {
	                of0 = of;
	                for (int i=0; i<basiscnt; i++)
	                  {
	                         x0[i] = testvector[i];
	                  }
	                for (int i=0; i < numbasis; i++)
	                  {
	                         x0perc[i] = perc[i];
	                         x0raw[i]=raw[i];
	                  }
	                red++;
	                rep=0;
	             }

	     else
	       {
	          // If OF is increased, accept new position with probability
	          // dependent on size of increase

	          // Change in objective function
	          dof = of - of0;
	          inx++;

	          // Apply the Metropolis test
	          double plim = Math.exp((double) -1 * dof / t);
	          double p = rnums.nextDouble();
	          if (p < 0) p = p * (double) -1;
	          if (p < plim)
	             {
	                of0 = of;
	                for (int i=0; i<basiscnt; i++)
	                  {
	                         x0[i] = testvector[i];
	                  }
	                for (int i=0; i < numbasis; i++)
	                  {
	                        x0perc[i] = perc[i];
	                        x0raw[i] = raw[i];
	                  }

	                inc++;
	                rep = 0;
	              }
	          else
	              {
	                rep++;
	              }
	       }


	   // Decrease temperature after LTRAN trials or after MACCS acceptances
	   if (((inc+red) > maccs) || (its > ltran))
	     {
	       rat = (double) inc / (double) inx;
	       incs = incs + inc;
	       reds = reds + red;
	       its = 0;
	       inc = 0;
	       inx = 0;
	       red = 0;

	       // Check for convergence - no absolute reduction at this temperature
	       // and acceptance ratio less than ratmin
	       //if ((ard == ards) && (rat < (double) 0.1)) break OUT;
	       if ((ard == ards) && (rat < ratmin)) break OUT;
	       ards = ard;

	       // Output monitoring information
	       //outputArea.appendText(trial+"  "+ of0+"  "+ ofmin+"  "+ incs+"  "+ reds+"  "+ard+"  "+t+"  "+rat+"\n");

           MessageFormat mf = new MessageFormat(
		      "{0,number, ######}\t{1,number,####.####}\t{2,number,####.####}\t{3,number,######}\t{4,number,######}\t{5,number,######}\t{6,number,##.###}\t{7,number,##.##}");
	       Object[] arguments = {new Integer(trial), new Double(-1*of0), new Double(-1*ofmin), new Integer(incs),new Integer(reds),new Integer(ard),new Double(t),new Double(rat)};
	       outputArea.appendText(mf.format(arguments)+"\n");



	       // Temperature is actually changed here
	       t = alfa * t;
	       //t = t / Math.log(trial);  // Boltzman annealing schedule
	    }
	   // End of one iteration
	 }

	 // break OUT point, Final output information


	 //outputArea.appendText(trial+"  "+of0+"  "+ofmin+"  "+incs+"  "+reds+"  "+ard+"  "+t+"  "+rat+"\n");
     MessageFormat mf = new MessageFormat(
      "{0,number, ######}\t{1,number,####.####}\t{2,number,####.####}\t{3,number,######}\t{4,number,######}\t{5,number,######}\t{6,number,##.###}\t{7,number,##.##}");
     Object[] arguments = {new Integer(trial), new Double(-1*of0), new Double(-1*ofmin), new Integer(incs),new Integer(reds),new Integer(ard),new Double(t),new Double(rat)};
     outputArea.appendText(mf.format(arguments)+"\n");


	 // Output best solution found

	 outputArea.appendText("Best solution found in "+trial+" total permutations\n");
	 //outputArea.appendText("Final score (-1 * correlation coefficient): "+ofmin+"\n");
	 MessageFormat mffs = new MessageFormat(
	   "Final score (Pearson correlation coefficient) = {0,number, #.#####}");
	 Object[] argumentsfs = {new Double(-1 * ofmin)};
	 outputArea.appendText(mffs.format(argumentsfs)+"\n");


	 //for ($i=0; $i< $basiscnt; $i++)
	 //  {
	 //       print "$xmin[$i] ";
	 //  }
	 MessageFormat mfbs = new MessageFormat(
	   "\t% {0}\t(basis vector {1,number,####}) =\t{2,number, ###.###}");
	 for (int i=0; i<numbasis; i++)
	    {
			Object[] argumentsbs = {new String(basislabel[i]), new Integer(i), new Double(100*xminperc[i])};
	 		outputArea.appendText(mfbs.format(argumentsbs)+"\n");
	        //outputArea.appendText("\t% "+basislabel[i]+"\t(basis vector "+i+") =\t"+(100*xminperc[i])+"\n");
        }
     outputArea.appendText("----------------------------------------------------------------------------------------------------------\n");
 	 //outputArea.repaint(1);
 }


   public void showAboutBox() {
	  String ABOUTMSG = "                            Expression Deconvolution \n== Reconstructing cell populations from DNA microarray data ==\n \nJanuary 2003 -- written in Java 2 v. 1.4\n\nEdward Marcotte\nInstitute for Cellular and Molecular Biology\nCenter for Computational Biology and Bioinformatics\nUniversity of Texas, Austin\nAustin, Texas, 78712";
      JOptionPane.showMessageDialog(this, ABOUTMSG);
   }

//}
}